'use strict';

let Process;

const Connections = new Map();
const Channels = new Map();

class ConnectionHandle
{
	constructor( handle )
	{
		this.handle = handle;
	}

	send( message )
	{
		console.log( 'Sending message to ', this.handle, message );

		if( this.handle.channel )
		{
			console.log( 'Channel ' + this.handle.channel + ' : ', Channels.get( this.handle.channel ) );
		}
	}
}

module.exports = class Connection
{
	constructor( id )
	{
		this.id = id;

		if( !Process )
		{
			Process = require( '../process.child' );
		}

		Process.emit({ event: 'connected', connection: { id } });

		Connections.set( id, this );
	}

	// TODO trigger event to master
	join( channels )
	{
		if( !Array.isArray( channels ) ){ channels = [ channels ]; }

		for( let channel of channels )
		{
			let channel_connections = Channels.get( channel );

			if( !channel_connections )
			{
				Channels.set( channel, channel_connections = new Set() );
			}

			channel_connections.add( this.id );
		}
	}

	leave( channels )
	{
		if( !Array.isArray( channels ) ){ channels = [ channels ]; }

		for( let channel of channels )
		{
			let channel_connections = Channels.get( channel );

			if( channel_connections )
			{
				channel_connections.delete( this.id );
			}
		}
	}

	destroy()
	{
		if( Connections.get( this.id ) === this )
		{
			Connections.delete( this.id );

			// TODO maybe remember joined channels
			for( let channel_connections of Channels.values() )
			{
				channel_connections.delete( this.id );
			}

			Process.emit({ event: 'disconnected', connection: { id: this.id } });
		}
	}

	static get( id )
	{
		return new ConnectionHandle({ id });
	}

	static any( channel )
	{
		return new ConnectionHandle({ channel: channel, connections: 1 });
	}

	static all( channel )
	{
		return new ConnectionHandle({ channel: channel, connections: Infinity });
	}

// send :message reliable (true/false)
}
